@@ -6,8 +6,46 @@ import string
 import autonetkit.anm
 import autonetkit.log as log
 
+class NidbEncoder(json.JSONEncoder):
+    #TODO: would be better to build this as a graph and then use normal JSON serialization
+    """Recursive handling and formatting for BGP Policy export in JSON format"""
+    def default(self, obj):
+
+        if isinstance(obj, netaddr.IPAddress):
+            return str(obj)
+        if isinstance(obj, netaddr.IPNetwork):
+            return str(obj)
+
+        if isinstance(obj, autonetkit.nidb.NIDB):
+            #TODO: add documentation about serializing anm nodes
+            return {'name': 'nidb',
+                    'children': list(obj.nodes("is_l3device"))}
+
+        if isinstance(obj, autonetkit.nidb.nidb_node):
+            return {'name': obj.id,
+                    'children': list(obj.get_interfaces())}
+            #TODO: also need to return non interface blocks
+# use keys of node
+            return str(obj)
+
+        if isinstance(obj, autonetkit.nidb.overlay_interface):
+            return [{'name': obj.id,
+                'children': list(obj.dict())}]
+        #return str(obj)
+
+        if isinstance(obj, dict):
+            return str(obj) #TODO: need to handle as list of children
+            #return [{"name": k, "value": v} for k, v in obj.items
+
+        return json.JSONEncoder.default(self, obj)
+
+def ank_nidb_tree(nidb, indent = 4):
+    data = json.dumps(nidb, cls=NidbEncoder, indent = indent)
+    return data
+
 class AnkEncoder(json.JSONEncoder):
     """Handles netaddr objects by converting to string form"""
+    #TODO: look at using skipkeys = True to skip non-basic type keys (can we warn on this?)
     def default(self, obj):
         if isinstance(obj, set):
             return str(obj)
@@ -30,18 +68,17 @@ class AnkEncoder(json.JSONEncoder):
             #TODO: add documentation about serializing anm nodes
             log.debug("%s is nidb nidb_node_category. Use attribute rather than object in compiler." % obj)
             return str(obj)
-
-        
+        if isinstance(obj, nx.classes.Graph):
+            return json_graph.node_link_data(obj)
 
         return json.JSONEncoder.default(self, obj)
 
 def ank_json_dumps(graph, indent = 4):
     data =  json_graph.node_link_data(graph)
 #TODO: use regex to convert IPAddress and IPNetwork back to respective form in decoder
-    data = json.dumps(data, cls=AnkEncoder, indent = indent)
+    data = json.dumps(data, cls=AnkEncoder, indent = indent, sort_keys = True)
     return data
 
-
 def string_to_netaddr(val):
     retval = None
     dot_count = string.count(val, ".")
@@ -110,9 +147,16 @@ def jsonify_anm(anm):
     return json.dumps(anm_json)
 
 
-def jsonify_anm_with_graphics(anm):
+def shortened_interface(name):
+    """Condenses interface name. Not canonical - mainly for brevity"""
+    name = name.replace("GigabitEthernet", "ge")
+    name = name.replace("0/0/0/", "")
+    return name
+
+def jsonify_anm_with_graphics(anm, nidb = None):
     """ Returns a dictionary of json-ified overlay graphs, with graphics data appended to each overlay"""
     anm_json = {}
+    test_anm_data = {}
     graphics_graph = anm["graphics"]._graph.copy()
     for overlay_id in anm.overlays():
         OverlayGraph = anm[overlay_id]._graph.copy()
@@ -134,6 +178,26 @@ def jsonify_anm_with_graphics(anm):
             except KeyError:
                 pass
 
+            if nidb:
+                nidb_graph = nidb._graph
+                if n in nidb:
+                    nidb_node_data = nidb_graph.node[n]
+                    try:
+                        #TODO: check why not all nodes have _interfaces initialised
+                        overlay_interfaces = OverlayGraph.node[n]["_interfaces"]
+                    except KeyError:
+                        continue # skip copying interface data for this node
+
+                    for interface_id in overlay_interfaces.keys():
+                        try:
+                            nidb_interface_id = nidb_node_data['_interfaces'][interface_id]['id']
+                        except KeyError:
+                            #TODO: check why arrive here - something not initialised?
+                            continue
+                        OverlayGraph.node[n]['_interfaces'][interface_id]['id'] = nidb_interface_id
+                        id_brief = shortened_interface(nidb_interface_id)
+                        OverlayGraph.node[n]['_interfaces'][interface_id]['id_brief'] = id_brief
+
 #TODO: combine these, and round as necessary
         x = (OverlayGraph.node[n]['x'] for n in OverlayGraph)
         y = (OverlayGraph.node[n]['y'] for n in OverlayGraph)
@@ -150,9 +214,15 @@ def jsonify_anm_with_graphics(anm):
             OverlayGraph.node[n]['y'] += - y_min
 
         anm_json[overlay_id] = ank_json_dumps(OverlayGraph)
-    return anm_json
+        test_anm_data[overlay_id] = OverlayGraph
 
-def jsonify_nidb(nidb):
+    if nidb:
+        test_anm_data['nidb'] = prepare_nidb(nidb)
+
+    result = json.dumps(test_anm_data, cls=AnkEncoder, indent = 4, sort_keys = True)
+    return result
+
+def prepare_nidb(nidb):
     graph = nidb._graph
     for node in graph:
         graph.node[node]['x'] = graph.node[node]['graphics']['x']
@@ -160,6 +230,14 @@ def jsonify_nidb(nidb):
         graph.node[node]['device_type'] = graph.node[node]['graphics']['device_type']
         graph.node[node]['device_subtype'] = graph.node[node]['graphics']['device_subtype']
 
+        for interface_index in graph.node[node]['_interfaces']:
+            try:
+                interface_id = graph.node[node]["_interfaces"][interface_index]['id']
+            except KeyError: # interface doesn't exist, eg for a lan segment
+                interface_id = ""
+            id_brief = shortened_interface(interface_id)
+            graph.node[node]["_interfaces"][interface_index]['id_brief'] = id_brief
+
     x = (graph.node[n]['x'] for n in graph)
     y = (graph.node[n]['y'] for n in graph)
     x_min = min(x)
@@ -168,13 +246,12 @@ def jsonify_nidb(nidb):
         graph.node[n]['x'] += - x_min
         graph.node[n]['y'] += - y_min
 
+    return graph
+
+def jsonify_nidb(nidb):
+    graph = prepare_nidb(nidb)
     data = ank_json_dumps(graph)
     return data
 
-def dumps(anm, nidb = None):
-    data = jsonify_anm_with_graphics(anm)
-    if nidb:
-        data['nidb'] = jsonify_nidb(nidb)
-#TODO: need to update messaging format when have nidb also (as 'anm': won't be correct)
-    json_data = json.dumps({'anm': data})
-    return json_data
+def dumps(anm, nidb = None, indent = 4):
+    return jsonify_anm_with_graphics(anm, nidb)
