@@ -4,7 +4,8 @@ import autonetkit.log as log
 import autonetkit.ank_json
 import netaddr
 
-messaging = autonetkit.ank_messaging.AnkMessaging()
+messaging = autonetkit.ank_messaging
+
 
 #TODO: allow slack in allocations: both for ASN (group level), and for collision domains to allow new nodes to be easily added
 
@@ -35,6 +36,7 @@ def allocate_ips(G_ip):
 
     loopback_blocks = {}
     infra_blocks = {}
+    secondary_loopback_blocks = {}
 
 #TODO: check if need to do network address... possibly only for loopback_pool and infra_pool so maps to asn
 
@@ -44,14 +46,18 @@ def allocate_ips(G_ip):
     [global_pool.next() for i in range(9)] # consume generator to start infra at "a", loopbacks at "b"
     loopback_pool = global_pool.next().subnet(80)
     infra_pool = global_pool.next().subnet(80)
+    secondary_loopback_pool = global_pool.next().subnet(80)
 
+    # consume the first address as it is the network address
     loopback_network = loopback_pool.next() # network address
     infra_network = infra_pool.next() # network address
+    secondary_loopback_network = secondary_loopback_pool.next() # network address
 
     unique_asns = set(n.asn for n in G_ip)
     for asn in sorted(unique_asns):
         loopback_blocks[asn] = loopback_pool.next()
         infra_blocks[asn] = infra_pool.next()
+        secondary_loopback_blocks[asn] = secondary_loopback_network.next()
 
     for asn, devices in G_ip.groupby("asn").items():
         subnets = infra_blocks[asn].subnet(96)
@@ -86,6 +92,17 @@ def allocate_ips(G_ip):
         for host in sorted(l3hosts):
             host.loopback = loopback_hosts.next()
 
+        routers = [n for n in l3hosts if n.is_router] # filter
+        secondary_loopbacks = [i for n in routers
+                for i in n.loopback_interfaces
+                if not i.is_loopback_zero]
+
+        secondary_loopback_hosts = secondary_loopback_blocks[asn].iter_hosts()
+        secondary_loopback_hosts.next() # drop .0 as a host address (valid but can be confusing)
+        for interface in sorted(secondary_loopbacks):
+            #TODO: check sorting is first node, then interface
+            interface.loopback = secondary_loopback_hosts.next()
+
     # Store allocations for routing advertisement
 # convert blocks from being {asn: block} to {asn: [block]} for consistency with ipv4, and scalability with compiler if multiple blocks in future
     G_ip.data.infra_blocks = dict((asn, [subnet]) for asn, subnet in infra_blocks.items())
@@ -136,9 +153,7 @@ def allocate_ips(G_ip):
             }
 
     jsontree = json.dumps(total_tree, cls=autonetkit.ank_json.AnkEncoder, indent = 4)
-    
-    body = json.dumps({"ip_allocations": jsontree})
-    #messaging.publish_compressed("www", "client", body)
+    messaging.publish_data(jsontree, "ip_allocations")
 
 
 #TODO: need to update with loopbacks if wish to advertise also - or subdivide blocks?
